@@ -1,29 +1,24 @@
-/***
-  First set of functions in this file are part of systemd, and were
-  copied to util-linux at August 2013.
-
-  Copyright 2010 Lennart Poettering
-  Copyright (C) 2014 Karel Zak <kzak@redhat.com>
-
-  systemd is free software; you can redistribute it and/or modify it
-  under the terms of the GNU Lesser General Public License as published by
-  the Free Software Foundation; either version 2.1 of the License, or
-  (at your option) any later version.
-
-  systemd is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-  Lesser General Public License for more details.
-
-  You should have received a copy of the GNU Lesser General Public License
-  along with systemd; If not, see <http://www.gnu.org/licenses/>.
-***/
+/*
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
+ * First set of functions in this file are part of systemd, and were
+ * copied to util-linux at August 2013.
+ *
+ * Copyright 2010 Lennart Poettering
+ * Copyright (C) 2014 Karel Zak <kzak@redhat.com>
+ *
+ * This is free software; you can redistribute it and/or modify it under the
+ * terms of the GNU Lesser General Public License as published by the Free
+ * Software Foundation; either version 2.1 of the License, or (at your option)
+ * any later version.
+ */
 #ifndef UTIL_LINUX_TIME_UTIL_H
 #define UTIL_LINUX_TIME_UTIL_H
 
 #include <stdio.h>
 #include <inttypes.h>
 #include <sys/time.h>
+#include <stdbool.h>
 
 typedef uint64_t usec_t;
 typedef uint64_t nsec_t;
@@ -62,8 +57,10 @@ enum {
 	ISO_TIMEZONE		= (1 << 2),
 	ISO_DOTUSEC		= (1 << 3),
 	ISO_COMMAUSEC		= (1 << 4),
-	ISO_T			= (1 << 5),
-	ISO_GMTIME		= (1 << 6),
+	ISO_DOTNSEC		= (1 << 5),
+	ISO_COMMANSEC		= (1 << 6),
+	ISO_T			= (1 << 7),
+	ISO_GMTIME		= (1 << 8),
 	ISO_TIMESTAMP		= ISO_DATE | ISO_TIME | ISO_TIMEZONE,
 	ISO_TIMESTAMP_T		= ISO_TIMESTAMP | ISO_T,
 	ISO_TIMESTAMP_DOT	= ISO_TIMESTAMP | ISO_DOTUSEC,
@@ -77,9 +74,11 @@ enum {
 #define CTIME_BUFSIZ	26
 #define ISO_BUFSIZ	42
 
-int strtimeval_iso(struct timeval *tv, int flags, char *buf, size_t bufsz);
-int strtm_iso(struct tm *tm, int flags, char *buf, size_t bufsz);
+int strtimeval_iso(const struct timeval *tv, int flags, char *buf, size_t bufsz);
+int strtm_iso(const struct tm *tm, int flags, char *buf, size_t bufsz);
 int strtime_iso(const time_t *t, int flags, char *buf, size_t bufsz);
+int strtimespec_iso(const struct timespec *t, int flags, char *buf, size_t bufsz);
+int strtimespec_relative(const struct timespec *ts, char *buf, size_t bufsz);
 
 #define UL_SHORTTIME_THISYEAR_HHMM (1 << 1)
 
@@ -89,4 +88,28 @@ int strtime_short(const time_t *t, struct timeval *now, int flags, char *buf, si
 extern time_t timegm(struct tm *tm);
 #endif
 
+static inline usec_t timeval_to_usec(const struct timeval *t)
+{
+	return t->tv_sec * USEC_PER_SEC + t->tv_usec;
+}
+
+static inline usec_t timespec_to_usec(const struct timespec *t)
+{
+	return t->tv_sec * USEC_PER_SEC + t->tv_nsec / NSEC_PER_USEC;
+}
+
+static inline struct timeval usec_to_timeval(usec_t t)
+{
+	struct timeval r = {
+		.tv_sec = t / USEC_PER_SEC,
+		.tv_usec = t % USEC_PER_SEC,
+	};
+	return r;
+}
+
+static inline bool is_timespecset(const struct timespec *t)
+{
+	return t->tv_sec || t->tv_nsec;
+}
+
 #endif /* UTIL_LINUX_TIME_UTIL_H */
