@@ -1035,7 +1035,9 @@ int fdisk_info_new_partition(
 }
 
 #ifdef TEST_PROGRAM
-static int test_ranges(struct fdisk_test *ts, int argc, char *argv[])
+static int test_ranges(struct fdisk_test *ts __attribute__((unused)),
+		       int argc __attribute__((unused)),
+		       char *argv[] __attribute__((unused)))
 {
 	/*                1  -  3,       6,    8, 9,   11    13 */
 	size_t nums[] = { 1, 1, 1, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1 };
