@@ -650,8 +650,8 @@ int mnt_table_get_root_fs(struct libmnt_table *tb, struct libmnt_fs **root)
  * @parent: parental FS
  * @chld: NULL or returns the next child filesystem
  *
- * Note that filesystems are returned in the order of mounting (according to
- * IDs in /proc/self/mountinfo).
+ * Since version 2.40, the filesystems are returned in the order specified by
+ * @itr. In the old versions the derection is always MNT_ITER_FORWARD.
  *
  * Returns: 0 on success, negative number in case of error or 1 at the end of list.
  */
@@ -660,6 +660,7 @@ int mnt_table_next_child_fs(struct libmnt_table *tb, struct libmnt_iter *itr,
 {
 	struct libmnt_fs *fs, *chfs = NULL;
 	int parent_id, lastchld_id = 0, chld_id = 0;
+	int direction;
 
 	if (!tb || !itr || !parent || !is_mountinfo(tb))
 		return -EINVAL;
@@ -667,6 +668,7 @@ int mnt_table_next_child_fs(struct libmnt_table *tb, struct libmnt_iter *itr,
 	DBG(TAB, ul_debugobj(tb, "lookup next child of '%s'",
 				mnt_fs_get_target(parent)));
 	parent_id = mnt_fs_get_id(parent);
+	direction = mnt_iter_get_direction(itr);
 
 	/* get ID of the previously returned child */
 	if (itr->head && itr->p != itr->head) {
@@ -675,7 +677,7 @@ int mnt_table_next_child_fs(struct libmnt_table *tb, struct libmnt_iter *itr,
 		lastchld_id = mnt_fs_get_id(fs);
 	}
 
-	mnt_reset_iter(itr, MNT_ITER_FORWARD);
+	mnt_reset_iter(itr, direction);
 	while (mnt_table_next_fs(tb, itr, &fs) == 0) {
 		int id;
 
@@ -689,10 +691,20 @@ int mnt_table_next_child_fs(struct libmnt_table *tb, struct libmnt_iter *itr,
 		if (id == parent_id)
 			continue;
 
-		if ((!lastchld_id || id > lastchld_id) &&
-		    (!chfs || id < chld_id)) {
-			chfs = fs;
-			chld_id = id;
+		if (direction == MNT_ITER_FORWARD) {
+			/* return in the order of mounting */
+			if ((!lastchld_id || id > lastchld_id) &&
+			    (!chfs || id < chld_id)) {
+				chfs = fs;
+				chld_id = id;
+			}
+		} else {
+			/* return last child first */
+			if ((!lastchld_id || id < lastchld_id) &&
+			    (!chfs || id > chld_id)) {
+				chfs = fs;
+				chld_id = id;
+			}
 		}
 	}
 
@@ -1913,7 +1925,8 @@ int mnt_table_is_fs_mounted(struct libmnt_table *tb, struct libmnt_fs *fstab_fs)
 #ifdef TEST_PROGRAM
 #include "pathnames.h"
 
-static int parser_errcb(struct libmnt_table *tb, const char *filename, int line)
+static int parser_errcb(struct libmnt_table *tb __attribute__((unused)),
+			const char *filename, int line)
 {
 	fprintf(stderr, "%s:%d: parse error\n", filename, line);
 
@@ -1942,12 +1955,16 @@ err:
 	return NULL;
 }
 
-static int test_copy_fs(struct libmnt_test *ts, int argc, char *argv[])
+static int test_copy_fs(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	struct libmnt_table *tb;
 	struct libmnt_fs *fs;
 	int rc = -1;
 
+	if (argc != 2)
+		return -1;
+
 	tb = create_table(argv[1], FALSE);
 	if (!tb)
 		return -1;
@@ -1972,7 +1989,8 @@ done:
 	return rc;
 }
 
-static int test_parse(struct libmnt_test *ts, int argc, char *argv[])
+static int test_parse(struct libmnt_test *ts __attribute__((unused)),
+		      int argc, char *argv[])
 {
 	struct libmnt_table *tb = NULL;
 	struct libmnt_iter *itr = NULL;
@@ -2008,7 +2026,8 @@ done:
 	return rc;
 }
 
-static int test_find_idx(struct libmnt_test *ts, int argc, char *argv[])
+static int test_find_idx(struct libmnt_test *ts __attribute__((unused)),
+			 int argc, char *argv[])
 {
 	struct libmnt_table *tb;
 	struct libmnt_fs *fs = NULL;
@@ -2053,7 +2072,8 @@ done:
 	return rc;
 }
 
-static int test_find(struct libmnt_test *ts, int argc, char *argv[], int dr)
+static int test_find(struct libmnt_test *ts __attribute__((unused)),
+		     int argc, char *argv[], int dr)
 {
 	struct libmnt_table *tb;
 	struct libmnt_fs *fs = NULL;
@@ -2095,23 +2115,29 @@ done:
 	return rc;
 }
 
-static int test_find_bw(struct libmnt_test *ts, int argc, char *argv[])
+static int test_find_bw(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	return test_find(ts, argc, argv, MNT_ITER_BACKWARD);
 }
 
-static int test_find_fw(struct libmnt_test *ts, int argc, char *argv[])
+static int test_find_fw(struct libmnt_test *ts __attribute__((unused)),
+			int argc, char *argv[])
 {
 	return test_find(ts, argc, argv, MNT_ITER_FORWARD);
 }
 
-static int test_find_pair(struct libmnt_test *ts, int argc, char *argv[])
+static int test_find_pair(struct libmnt_test *ts __attribute__((unused)),
+			  int argc, char *argv[])
 {
 	struct libmnt_table *tb;
 	struct libmnt_fs *fs;
 	struct libmnt_cache *mpc = NULL;
 	int rc = -1;
 
+	if (argc != 4)
+		return -1;
+
 	tb = create_table(argv[1], FALSE);
 	if (!tb)
 		return -1;
@@ -2132,13 +2158,17 @@ done:
 	return rc;
 }
 
-static int test_find_mountpoint(struct libmnt_test *ts, int argc, char *argv[])
+static int test_find_mountpoint(struct libmnt_test *ts __attribute__((unused)),
+				int argc, char *argv[])
 {
 	struct libmnt_table *tb;
 	struct libmnt_fs *fs;
 	struct libmnt_cache *mpc = NULL;
 	int rc = -1;
 
+	if (argc != 2)
+		return -1;
+
 	tb = mnt_new_table_from_file(_PATH_PROC_MOUNTINFO);
 	if (!tb)
 		return -1;
@@ -2159,13 +2189,17 @@ done:
 	return rc;
 }
 
-static int test_is_mounted(struct libmnt_test *ts, int argc, char *argv[])
+static int test_is_mounted(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
 	struct libmnt_table *tb = NULL, *fstab = NULL;
 	struct libmnt_fs *fs;
 	struct libmnt_iter *itr = NULL;
 	struct libmnt_cache *mpc = NULL;
 
+	if (argc != 2)
+		return -1;
+
 	tb = mnt_new_table_from_file("/proc/self/mountinfo");
 	if (!tb) {
 		fprintf(stderr, "failed to parse mountinfo\n");
@@ -2215,7 +2249,8 @@ static int test_uniq_cmp(struct libmnt_table *tb __attribute__((__unused__)),
 	return mnt_fs_streq_target(a, mnt_fs_get_target(b)) ? 0 : 1;
 }
 
-static int test_uniq(struct libmnt_test *ts, int argc, char *argv[])
+static int test_uniq(struct libmnt_test *ts __attribute__((unused)),
+		     int argc, char *argv[])
 {
 	struct libmnt_table *tb;
 	int rc = -1;
