@@ -340,6 +340,7 @@ int mnt_fstype_is_pseudofs(const char *type)
 		"none",
 		"nsfs",
 		"overlay",
+		"pidfs",
 		"pipefs",
 		"proc",
 		"pstore",
@@ -526,7 +527,7 @@ static int add_filesystem(char ***filesystems, char *name)
 
 	if (n == 0 || !((n + 1) % MYCHUNK)) {
 		size_t items = ((n + 1 + MYCHUNK) / MYCHUNK) * MYCHUNK;
-		char **x = realloc(*filesystems, items * sizeof(char *));
+		char **x = reallocarray(*filesystems, items, sizeof(char *));
 
 		if (!x)
 			goto err;
@@ -1032,7 +1033,7 @@ int mnt_open_uniq_filename(const char *filename, char **name)
 
 	rc = asprintf(&n, "%s.XXXXXX", filename);
 	if (rc <= 0)
-		return -errno;
+		return -ENOMEM;
 
 	/* This is for very old glibc and for compatibility with Posix, which says
 	 * nothing about mkstemp() mode. All sane glibc use secure mode (0600).
@@ -1134,7 +1135,7 @@ char *mnt_get_kernel_cmdline_option(const char *name)
 	int val = 0;
 	char *p, *res = NULL, *mem = NULL;
 	char buf[BUFSIZ];	/* see kernel include/asm-generic/setup.h: COMMAND_LINE_SIZE */
-	const char *path = _PATH_PROC_CMDLINE;
+	const char *path;
 
 	if (!name || !name[0])
 		return NULL;
@@ -1143,6 +1144,8 @@ char *mnt_get_kernel_cmdline_option(const char *name)
 	path = getenv("LIBMOUNT_KERNEL_CMDLINE");
 	if (!path)
 		path = _PATH_PROC_CMDLINE;
+#else
+	path = _PATH_PROC_CMDLINE;
 #endif
 	f = fopen(path, "r" UL_CLOEXECSTR);
 	if (!f)
@@ -1287,8 +1290,12 @@ done:
 }
 
 #ifdef TEST_PROGRAM
-static int test_match_fstype(struct libmnt_test *ts, int argc, char *argv[])
+static int test_match_fstype(struct libmnt_test *ts __attribute__((unused)),
+			     int argc, char *argv[])
 {
+	if (argc != 3)
+		return -1;
+
 	char *type = argv[1];
 	char *pattern = argv[2];
 
@@ -1296,8 +1303,12 @@ static int test_match_fstype(struct libmnt_test *ts, int argc, char *argv[])
 	return 0;
 }
 
-static int test_match_options(struct libmnt_test *ts, int argc, char *argv[])
+static int test_match_options(struct libmnt_test *ts __attribute__((unused)),
+			      int argc, char *argv[])
 {
+	if (argc != 3)
+		return -1;
+
 	char *optstr = argv[1];
 	char *pattern = argv[2];
 
@@ -1305,8 +1316,12 @@ static int test_match_options(struct libmnt_test *ts, int argc, char *argv[])
 	return 0;
 }
 
-static int test_startswith(struct libmnt_test *ts, int argc, char *argv[])
+static int test_startswith(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
+	if (argc != 3)
+		return -1;
+
 	char *optstr = argv[1];
 	char *pattern = argv[2];
 
@@ -1314,8 +1329,12 @@ static int test_startswith(struct libmnt_test *ts, int argc, char *argv[])
 	return 0;
 }
 
-static int test_endswith(struct libmnt_test *ts, int argc, char *argv[])
+static int test_endswith(struct libmnt_test *ts __attribute__((unused)),
+			 int argc, char *argv[])
 {
+	if (argc != 3)
+		return -1;
+
 	char *optstr = argv[1];
 	char *pattern = argv[2];
 
@@ -1323,8 +1342,12 @@ static int test_endswith(struct libmnt_test *ts, int argc, char *argv[])
 	return 0;
 }
 
-static int test_mountpoint(struct libmnt_test *ts, int argc, char *argv[])
+static int test_mountpoint(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	char *path = canonicalize_path(argv[1]),
 	     *mnt = path ? mnt_get_mountpoint(path) :  NULL;
 
@@ -1334,13 +1357,17 @@ static int test_mountpoint(struct libmnt_test *ts, int argc, char *argv[])
 	return 0;
 }
 
-static int test_filesystems(struct libmnt_test *ts, int argc, char *argv[])
+static int test_filesystems(struct libmnt_test *ts __attribute__((unused)),
+			    int argc, char *argv[])
 {
 	char **filesystems = NULL;
 	int rc;
 
-	rc = mnt_get_filesystems(&filesystems, argc ? argv[1] : NULL);
-	if (!rc) {
+	if (argc != 1 && argc != 2)
+		return -1;
+
+	rc = mnt_get_filesystems(&filesystems, argc == 2 ? argv[1] : NULL);
+	if (!rc && filesystems) {
 		char **p;
 		for (p = filesystems; *p; p++)
 			printf("%s\n", *p);
@@ -1349,8 +1376,12 @@ static int test_filesystems(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_chdir(struct libmnt_test *ts, int argc, char *argv[])
+static int test_chdir(struct libmnt_test *ts __attribute__((unused)),
+		      int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	int rc;
 	char *path = canonicalize_path(argv[1]),
 	     *last = NULL;
@@ -1368,8 +1399,12 @@ static int test_chdir(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_kernel_cmdline(struct libmnt_test *ts, int argc, char *argv[])
+static int test_kernel_cmdline(struct libmnt_test *ts __attribute__((unused)),
+			       int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	char *name = argv[1];
 	char *res;
 
@@ -1387,7 +1422,8 @@ static int test_kernel_cmdline(struct libmnt_test *ts, int argc, char *argv[])
 }
 
 
-static int test_guess_root(struct libmnt_test *ts, int argc, char *argv[])
+static int test_guess_root(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
 	int rc;
 	char *real;
@@ -1413,10 +1449,14 @@ static int test_guess_root(struct libmnt_test *ts, int argc, char *argv[])
 	return 0;
 }
 
-static int test_mkdir(struct libmnt_test *ts, int argc, char *argv[])
+static int test_mkdir(struct libmnt_test *ts __attribute__((unused)),
+	 	      int argc, char *argv[])
 {
 	int rc;
 
+	if (argc != 2)
+		return -1;
+
 	rc = ul_mkdir_p(argv[1], S_IRWXU |
 			 S_IRGRP | S_IXGRP |
 			 S_IROTH | S_IXOTH);
@@ -1425,11 +1465,15 @@ static int test_mkdir(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int test_statfs_type(struct libmnt_test *ts, int argc, char *argv[])
+static int test_statfs_type(struct libmnt_test *ts __attribute__((unused)),
+			    int argc, char *argv[])
 {
 	struct statfs vfs;
 	int rc;
 
+	if (argc != 2)
+		return -1;
+
 	rc = statfs(argv[1], &vfs);
 	if (rc)
 		printf("%s: statfs failed: %m\n", argv[1]);
@@ -1440,8 +1484,12 @@ static int test_statfs_type(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int tests_parse_uid(struct libmnt_test *ts, int argc, char *argv[])
+static int tests_parse_uid(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	char *str = argv[1];
 	uid_t uid = (uid_t) -1;
 	int rc;
@@ -1455,8 +1503,12 @@ static int tests_parse_uid(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int tests_parse_gid(struct libmnt_test *ts, int argc, char *argv[])
+static int tests_parse_gid(struct libmnt_test *ts __attribute__((unused)),
+			   int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	char *str = argv[1];
 	gid_t gid = (gid_t) -1;
 	int rc;
@@ -1470,8 +1522,12 @@ static int tests_parse_gid(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int tests_parse_mode(struct libmnt_test *ts, int argc, char *argv[])
+static int tests_parse_mode(struct libmnt_test *ts __attribute__((unused)),
+			    int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	char *str = argv[1];
 	mode_t mod = (mode_t) -1;
 	int rc;
@@ -1488,8 +1544,12 @@ static int tests_parse_mode(struct libmnt_test *ts, int argc, char *argv[])
 	return rc;
 }
 
-static int tests_stat(struct libmnt_test *ts, int argc, char *argv[])
+static int tests_stat(struct libmnt_test *ts __attribute__((unused)),
+		      int argc, char *argv[])
 {
+	if (argc != 2)
+		return -1;
+
 	char *path = argv[1];
 	struct stat st;
 	int rc;
