@@ -19,10 +19,6 @@
  * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include "xalloc.h"
-#include "nls.h"
-#include "libsmartcols.h"
-
 #include "lsfd.h"
 
 static struct list_head partitions;
@@ -50,7 +46,7 @@ static bool bdev_fill_column(struct proc *proc __attribute__((__unused__)),
 	case COL_BLKDRV:
 		devdrv = get_blkdrv(major(file->stat.st_rdev));
 		if (devdrv)
-			str = strdup(devdrv);
+			str = xstrdup(devdrv);
 		else
 			xasprintf(&str, "%u",
 				  major(file->stat.st_rdev));
@@ -64,7 +60,7 @@ static bool bdev_fill_column(struct proc *proc __attribute__((__unused__)),
 	case COL_PARTITION:
 		partition = get_partition(file->stat.st_rdev);
 		if (partition) {
-			str = strdup(partition);
+			str = xstrdup(partition);
 			break;
 		}
 		devdrv = get_blkdrv(major(file->stat.st_rdev));
