@@ -48,6 +48,9 @@
 #include <string.h>
 #include <ctype.h>
 #include <assert.h>
+#ifdef HAVE_FNMATCH
+# include <fnmatch.h>
+#endif
 
 #include "xalloc.h"
 #include "nls.h"
@@ -75,7 +78,8 @@ UL_DEBUG_DEFINE_MASKNAMES(whereis) = UL_DEBUG_EMPTY_MASKNAMES;
 #define UL_DEBUG_CURRENT_MASK	UL_DEBUG_MASK(whereis)
 #include "debugobj.h"
 
-static char uflag = 0;
+static char uflag;
+static char use_glob;
 
 /* supported types */
 enum {
@@ -212,11 +216,12 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -S <dirs>  define sources lookup path\n"), out);
 	fputs(_(" -f         terminate <dirs> argument list\n"), out);
 	fputs(_(" -u         search for unusual entries\n"), out);
+	fputs(_(" -g         interpret name as glob (pathnames pattern)\n"), out);
 	fputs(_(" -l         output effective lookup paths\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(16));
-	printf(USAGE_MAN_TAIL("whereis(1)"));
+	fprintf(out, USAGE_HELP_OPTIONS(16));
+	fprintf(out, USAGE_MAN_TAIL("whereis(1)"));
 	exit(EXIT_SUCCESS);
 }
 
@@ -394,13 +399,20 @@ static void free_dirlist(struct wh_dirlist **ls0, int type)
 
 static int filename_equal(const char *cp, const char *dp, int type)
 {
-	int i = strlen(dp);
+	size_t i;
 
 	DBG(SEARCH, ul_debug("compare '%s' and '%s'", cp, dp));
 
+#ifdef HAVE_FNMATCH
+	if (use_glob)
+		return fnmatch(cp, dp, 0) == 0;
+#endif
 	if (type & SRC_DIR &&
 	    dp[0] == 's' && dp[1] == '.' && filename_equal(cp, dp + 2, type))
 		return 1;
+
+	i = strlen(dp);
+
 	if (type & MAN_DIR) {
 		if (i > 1 && !strcmp(dp + i - 2, ".Z"))
 			i -= 2;
@@ -635,6 +647,9 @@ int main(int argc, char **argv)
 			case 'l':
 				list_dirlist(ls);
 				break;
+			case 'g':
+				use_glob = 1;
+				break;
 
 			case 'V':
 				print_version(EXIT_SUCCESS);
