@@ -90,9 +90,9 @@ static void __attribute__((__noreturn__)) usage(void)
 	fputs(_(" -v, --verbose        display status information\n"), out);
 
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(22));
+	fprintf(out, USAGE_HELP_OPTIONS(22));
 
-	printf(USAGE_MAN_TAIL("chrt(1)"));
+	fprintf(out, USAGE_MAN_TAIL("chrt(1)"));
 	exit(EXIT_SUCCESS);
 }
 
@@ -485,6 +485,8 @@ int main(int argc, char **argv)
 
 	if (!ctl->pid) {
 		argv += optind + 1;
+		if (strcmp(argv[0], "--") == 0)
+			argv++;
 		execvp(argv[0], argv);
 		errexec(argv[0]);
 	}
