@@ -1,6 +1,12 @@
 /*
  * SPDX-License-Identifier: GPL-2.0-or-later
  *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ *
  * i386 CMOS starts out with 14 bytes clock data alpha has something
  * similar, but with details depending on the machine type.
  *
@@ -369,7 +375,7 @@ static const char *get_device_path(void)
 	return NULL;
 }
 
-static struct clock_ops cmos_interface = {
+static const struct clock_ops cmos_interface = {
 	N_("Using direct ISA access to the clock"),
 	get_permissions_cmos,
 	read_hardware_clock_cmos,
@@ -381,7 +387,7 @@ static struct clock_ops cmos_interface = {
 /*
  * return &cmos if cmos clock present, NULL otherwise.
  */
-struct clock_ops *probe_for_cmos_clock(void)
+const struct clock_ops *probe_for_cmos_clock(void)
 {
 	return &cmos_interface;
 }
