@@ -94,6 +94,8 @@ static const struct id_part arm_part[] = {
     { 0xd81, "Cortex-A720" },
     { 0xd82, "Cortex-X4" },
     { 0xd84, "Neoverse-V3" },
+    { 0xd85, "Cortex-X925" },
+    { 0xd87, "Cortex-A725" },
     { 0xd8e, "Neoverse-N3" },
     { -1, "unknown" },
 };
