@@ -1,9 +1,5 @@
 /*
- * mountpoint(1) - see if a directory is a mountpoint
- *
- * This is libmount based reimplementation of the mountpoint(1)
- * from sysvinit project.
- *
+ * SPDX-License-Identifier: GPL-2.0-or-later
  *
  * Copyright (C) 2011 Red Hat, Inc. All rights reserved.
  * Written by Karel Zak <kzak@redhat.com>
@@ -13,16 +9,11 @@
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it would be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * mountpoint(1) - see if a directory is a mountpoint
  *
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ * This is libmount based reimplementation of the mountpoint(1)
+ * from sysvinit project.
  */
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -40,6 +31,8 @@
 #include "closestream.h"
 #include "pathnames.h"
 
+#define MOUNTPOINT_EXIT_NOMNT	32
+
 struct mountpoint_control {
 	char *path;
 	dev_t dev;
@@ -47,6 +40,7 @@ struct mountpoint_control {
 	unsigned int
 		dev_devno:1,
 		fs_devno:1,
+		nofollow:1,
 		quiet:1;
 };
 
@@ -123,11 +117,12 @@ static void __attribute__((__noreturn__)) usage(void)
 
 	fputs(USAGE_OPTIONS, out);
 	fputs(_(" -q, --quiet        quiet mode - don't print anything\n"
+		"     --nofollow     do not follow symlink\n"
 		" -d, --fs-devno     print maj:min device number of the filesystem\n"
 		" -x, --devno        print maj:min device number of the block device\n"), out);
 	fputs(USAGE_SEPARATOR, out);
-	printf(USAGE_HELP_OPTIONS(20));
-	printf(USAGE_MAN_TAIL("mountpoint(1)"));
+	fprintf(out, USAGE_HELP_OPTIONS(20));
+	fprintf(out, USAGE_MAN_TAIL("mountpoint(1)"));
 
 	exit(EXIT_SUCCESS);
 }
@@ -137,8 +132,13 @@ int main(int argc, char **argv)
 	int c;
 	struct mountpoint_control ctl = { NULL };
 
+	enum {
+		OPT_NOFOLLOW = CHAR_MAX + 1
+	};
+
 	static const struct option longopts[] = {
 		{ "quiet",    no_argument, NULL, 'q' },
+		{ "nofollow", no_argument, NULL, OPT_NOFOLLOW },
 		{ "fs-devno", no_argument, NULL, 'd' },
 		{ "devno",    no_argument, NULL, 'x' },
 		{ "help",     no_argument, NULL, 'h' },
@@ -159,6 +159,9 @@ int main(int argc, char **argv)
 		case 'q':
 			ctl.quiet = 1;
 			break;
+		case OPT_NOFOLLOW:
+			ctl.nofollow = 1;
+			break;
 		case 'd':
 			ctl.fs_devno = 1;
 			break;
@@ -179,24 +182,29 @@ int main(int argc, char **argv)
 		warnx(_("bad usage"));
 		errtryhelp(EXIT_FAILURE);
 	}
+	if (ctl.nofollow && ctl.dev_devno)
+		errx(EXIT_FAILURE, _("%s and %s are mutually exclusive"),
+		     "--devno", "--nofollow");
 
 	ctl.path = argv[optind];
-
-	if (stat(ctl.path, &ctl.st)) {
+	c = ctl.nofollow ? lstat(ctl.path, &ctl.st) : stat(ctl.path, &ctl.st);
+	if (c) {
 		if (!ctl.quiet)
 			err(EXIT_FAILURE, "%s", ctl.path);
 		return EXIT_FAILURE;
 	}
 	if (ctl.dev_devno)
-		return print_devno(&ctl) ? EXIT_FAILURE : EXIT_SUCCESS;
-	if (dir_to_device(&ctl)) {
+		return print_devno(&ctl) ? MOUNTPOINT_EXIT_NOMNT : EXIT_SUCCESS;
+
+	if ((ctl.nofollow && S_ISLNK(ctl.st.st_mode)) || dir_to_device(&ctl)) {
 		if (!ctl.quiet)
 			printf(_("%s is not a mountpoint\n"), ctl.path);
-		return EXIT_FAILURE;
+		return MOUNTPOINT_EXIT_NOMNT;
 	}
 	if (ctl.fs_devno)
 		printf("%u:%u\n", major(ctl.dev), minor(ctl.dev));
 	else if (!ctl.quiet)
 		printf(_("%s is a mountpoint\n"), ctl.path);
+
 	return EXIT_SUCCESS;
 }
